!  CONSTANT PARAMETERS
      real(8), parameter :: ZERO   =  0.0D+00
      real(8), parameter :: HALF   =  0.5D+00
      real(8), parameter :: ONE    =  1.0D+00
      real(8), parameter :: TWO    =  2.0D+00
!  PREFIX MINUS_ MEANS THE NEGATIVE SIGN
      real(8), parameter :: MINUS_HALF = -0.5D+00
      real(8), parameter :: MINUS_ONE  = -1.0D+00
      real(8), parameter :: MINUS_TWO  = -2.0D+00

