#!/bin/sh
NP=1

rm -f nfefn.data
for dir in 6.1 6.2 6.3 6.4 6.5 6.6 6.7 6.8 6.9 7.0 7.1 7.2 7.3 7.4 7.5 7.6 7.7 7.8 7.9 8.0;do
cd c$dir
mpiexec -n $NP ~/phase0_2014.03/bin/phase < /dev/null
echo -n $dir >> ../nfefn.data; echo -n; tail -1 nfefn.data >> ../nfefn.data
cd ../
done
